/**
 * Copyright 2006 (C) Bernd Bruegge, Timo Wolf - TU Muenchen
 * http://wwwbruegge.in.tum.de/Asteroids
 * 
 * This file is part of the Asteroids Teaching Design Pattern Project.
 * 
 * Asteroids is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * Asteroids is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Asteroids; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */
package de.tum.in.wwwbruegge.asteroids.ui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Iterator;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.KeyStroke;

import de.tum.in.wwwbruegge.asteroids.control.SpaceShuttleController;
import de.tum.in.wwwbruegge.asteroids.solidbodies.SpaceShuttle;

/**
 * @author Timo Wolf
 */
public class Game extends JFrame {

	private JPanel content = new JPanel(new BorderLayout(), true);

	public Game() {
		super("Asteroids");

		this.getContentPane().setLayout(new BorderLayout());

		content.add(ToolBar.getInstance(), BorderLayout.NORTH);
		content.add(GameBoard.getInstance(), BorderLayout.CENTER);
		JLabel bottomText = new JLabel(
				"Chair for Applied Software Engineering, TUM 2006",
				JLabel.CENTER);
		content.add(bottomText, BorderLayout.SOUTH);

		getContentPane().add(content, BorderLayout.CENTER);

		WindowAdapter windowAdapter = new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				Game.this.close();
			}
		};
		this.addWindowListener(windowAdapter);

		ActionListener exitMenuItemListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Game.this.close();
			}
		};
		JMenuItem exitMenuItem = new JMenuItem("Exit");
		exitMenuItem.addActionListener(exitMenuItemListener);

		JMenu fileMenu = new JMenu("File");
		fileMenu.add(exitMenuItem);

		JMenu controlMenu = new JMenu("Control");
		JMenuBar menuBar = new JMenuBar();
		menuBar.add(fileMenu);
		menuBar.add(controlMenu);
		setJMenuBar(menuBar);

		SpaceShuttle spaceshuttle = GameBoard.getInstance().getSpaceShuttle();
		SpaceShuttleController controller = new SpaceShuttleController(
				spaceshuttle);
		Iterator iter = controller.getActions().iterator();
		while (iter.hasNext()) {
			Action action = (Action) iter.next();
			controlMenu.add(action);
			Object binding = action.getValue(Action.NAME);
			KeyStroke ks = (KeyStroke) action.getValue(Action.ACCELERATOR_KEY);
			JComponent comp = getRootPane();
			comp.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
					.put(ks, binding);
			comp.getActionMap().put(binding, action);
		}

		JPanel controlpanel = new JPanel(new BorderLayout());
		JButton button = new JButton(controller.getSpeedUpAction());
		button.setFocusable(false);
		controlpanel.add(button, BorderLayout.NORTH);
		button = new JButton(controller.getSpeedDownAction());
		button.setFocusable(false);
		controlpanel.add(button, BorderLayout.SOUTH);
		button = new JButton(controller.getTurnLeftAction());
		button.setFocusable(false);
		controlpanel.add(button, BorderLayout.WEST);
		button = new JButton(controller.getTurnRightAction());
		button.setFocusable(false);
		controlpanel.add(button, BorderLayout.EAST);
		button = new JButton(controller.getFireRocketAction());
		button.setFocusable(false);
		controlpanel.add(button, BorderLayout.CENTER);

		InstrumentPanel instrumentpanel = new InstrumentPanel(GameBoard
				.getInstance().getSpaceShuttle());

		JPanel cockpitview = new JPanel(new BorderLayout());
		cockpitview.add(controlpanel, BorderLayout.NORTH);
		cockpitview.add(instrumentpanel, BorderLayout.CENTER);
		content.add(cockpitview, BorderLayout.EAST);
	}

	public Dimension getPreferredSize() {
		return Toolkit.getDefaultToolkit().getScreenSize();
	}

	private void close() {
		setVisible(false);
		System.exit(0);
	}
}
